from flask import Flask, jsonify, request, send_from_directory, Response, stream_with_context
from flask_cors import CORS
import json
from soundbridgefx import speakers

app = Flask(__name__)
CORS(app)
Speakers = speakers.Speakers()

@app.route("/api/v1/speakers/all", methods=["GET"])
def get_all_speakers():
    return Speakers.get_all_speakers()


@app.route("/api/v1/speakers/play/stream", methods=["POST"])
def play_stream():
    record = json.loads(request.data)
    response = Speakers.play_stream(record["selected"])
    return jsonify(response)


@app.route("/api/v1/speakers/play/stream", methods=["POST"])
def play_party():
    record = json.loads(request.data)
    response = Speakers.play_party(record["selected"])
    return jsonify(response)


@app.route("/api/v1/speakers/stop", methods=["POST"])
def stop():
    record = json.loads(request.data)
    response = Speakers.stop(record["selected"])
    return jsonify(response)


@app.route("/api/v1/v1/speakers/pause", methods=["POST"])
def pause():
    record = json.loads(request.data)
    response = Speakers.pause(record["selected"])
    return jsonify(response)


@app.route("/api/v1/speakers/volume", methods=["POST"])
def set_volume():
    record = json.loads(request.data)
    response = Speakers.set_volume(record["selected"])
    return jsonify(response)


@app.route("/stream_raw")
def stream():
    def generate():
        with open('stream.mp3', 'rb') as f:
            data = f.read(1024)
            while data:
                yield data
                data = f.read(1024)
    return Response(stream_with_context(generate()), mimetype='audio/mpeg')

"""Main route for serving the web app

Returns:
    _type_: _description_
"""
@app.route("/")
def base():
    return send_from_directory("web/prerendered/pages", "index.html")


"""Assets url

Returns:
    _type_: _description_
"""
@app.route("/<path:path>")
def home(path):
    return send_from_directory("web/client", path)

def main():
    # Run the app server on localhost:6207
    print(app.url_map)
    app.run(host='0.0.0.0',port=6207)



import logging
import os
_LOGGER = logging.getLogger(__name__)


class SonosSpeaker:

    def __init__(self, speaker):
        self.speaker = speaker
        self.stream_url = self.get_stream_url()
        self.player_name = self.speaker.player_name
        self.ip_addres = self.speaker.ip_address
        self.speaker_info = self.speaker.get_speaker_info()

        self.current_track_info = self.speaker.get_current_track_info()
        self.current_transport_info = self.speaker.get_current_transport_info()
        self.current_media_info = self.speaker.get_current_media_info()
        _LOGGER.debug(self.player_name, self.ip_addres)

    def get_stream_url(self):
        url = str(os.getenv('STREAM_URL'))+"/stream_raw"
        return url
        
    def __str__(self):
        return str(self.player_name)

    @property
    def update(self):
        self.current_track_info = self.speaker.get_current_track_info()
        self.current_transport_info = self.speaker.get_current_transport_info()
        self.current_media_info = self.speaker.get_current_media_info()

    def play_stream(self):
        _LOGGER.debug("going to play stream")
        self.speaker.play_uri(uri=self.stream_url,
                              title="SoundBridgeFx stream",
                              force_radio=True)

    def stop(self):
        self.speaker.stop()

    def pause(self):
        self.speaker.pause()

    def set_volume(self):
        vol = 50
        self.speaker.ramp_to_volume(vol)

    def reprJSON(self):
        return dict(player_name=self.player_name,
                    ip_addres=self.ip_addres,
                    current_track_info=self.current_track_info,
                    current_transport_info=self.current_transport_info,
                    current_media_info=self.current_media_info,
                    speaker_info=self.speaker_info,
                    type="sonos")




import json
from flask import jsonify, request
from soundbridgefx import ffmpeg, web_backend

app = web_backend.app
CVLC = ffmpeg.CLV()

##TODO refactor

@app.route("/api/v1/stream/speakers", methods=["POST"])
def get_all_stream_speakers():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)

@app.route("/api/v1/stream/microphones", methods=["POST"])
def get_all_stream_microphone():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)

@app.route("/api/v1/stream/speaker", methods=["POST"])
def get_all_stream_speaker():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)

app.route("/api/v1/stream/microphone", methods=["POST"])
def get_stream_microphone():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)

@app.route("/api/v1/stream/speaker", methods=["POST"])
def get_stream_speaker():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)

app.route("/api/v1/stream/microphone", methods=["POST"])
def set_stream_microphone():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)

app.route("/api/v1/stream/speaker", methods=["POST"])
def set_stream_speaker():
    record = json.loads(request.data)
    response = CVLC
    return jsonify(response)




import json
from flask import jsonify, request
from soundbridgefx import speakers, web_backend

app = web_backend.app
Speakers = speakers.Speakers()

@app.route("/api/v1/speakers/all", methods=["GET"])
def get_all_speakers():
    return Speakers.get_all_speakers()


@app.route("/api/v1/speakers/play/stream", methods=["POST"])
def play_stream():
    record = json.loads(request.data)
    response = Speakers.play_stream(record["selected"])
    return jsonify(response)


@app.route("/api/v1/speakers/play/stream", methods=["POST"])
def play_party():
    record = json.loads(request.data)
    response = Speakers.play_party(record["selected"])
    return jsonify(response)


@app.route("/api/v1/speakers/stop", methods=["POST"])
def stop():
    record = json.loads(request.data)
    response = Speakers.stop(record["selected"])
    return jsonify(response)


@app.route("/api/v1/v1/speakers/pause", methods=["POST"])
def pause():
    record = json.loads(request.data)
    response = Speakers.pause(record["selected"])
    return jsonify(response)


@app.route("/api/v1/speakers/volume", methods=["POST"])
def set_volume():
    record = json.loads(request.data)
    response = Speakers.set_volume(record["selected"])
    return jsonify(response)


import logging
import subprocess
import soundcard as sc

_LOGGER = logging.getLogger(__name__)

def main():
    try:
        p = subprocess.Popen(
            "librespot",
            stdout=subprocess.PIPE,
            shell=True)
    except:
        _LOGGER.error("Could not start cvlc")



import json
import logging
from soundbridgefx import (
    sonosspeakers
)

_LOGGER = logging.getLogger(__name__)

class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'reprJSON'):
            return obj.reprJSON()
        else:
            return json.JSONEncoder.default(self, obj)

        """
        A single sonos speaker, lowest level of abstraction.

        Returns:
            _type_: _description_
        """
class Speakers:
    def __init__(self):
        self.speakers = []
        self.sonos_speakers = sonosspeakers.SonosSpeakers()
        self.speakers.append(self.sonos_speakers)
        self.populate()

    def populate(self):
        json.dumps(self.sonos_speakers.reprJSON(), cls=ComplexEncoder)

    def reprJSON(self):
        return dict(all_speakers=self.speakers)

    def get_all_speakers(self):
        return json.dumps(self.reprJSON(), cls=ComplexEncoder)

    def play_stream(self, speakers):
        response = self.sonos_speakers.play_stream(speakers)
        return response

    def play_party(self, speakers):
        response = self.sonos_speakers.play_party(speakers)
        return response

    def pause(self, speakers):
        response = self.sonos_speakers.pause(speakers)
        return response

    def stop(self, speakers):
        response = self.sonos_speakers.stop(speakers)
        return response

    def set_volume(self, speakers):
        response = self.sonos_speakers.set_volume(speakers)
        return response



import soco
import logging
from soundbridgefx import (sonosspeaker)


_LOGGER = logging.getLogger(__name__)


"""
A class to hold all information about Sonos Speakers.
Please not the plural version of this, this class will scan for all sonos speakers on the network and manage them using this class.

Returns:
_type_: _description_
"""
class SonosSpeakers:

    def __init__(self):
        self.sonos_speakers_dict = []
        self.sonos_speakers = []
        self.N_speakers = None
        self.populate()

    """
    Populate the class with all sonos speakers on the network.
    """
    def populate(self):
        speakers = soco.discover()
        for speaker in speakers:
            sonos_speaker = sonosspeaker.SonosSpeaker(speaker)
            my_dict = {
                "player_name": sonos_speaker.player_name,
                "object": sonos_speaker
            }
            self.sonos_speakers_dict.append(my_dict)
            self.sonos_speakers.append(sonos_speaker)

        self.N_speakers = self.sonos_speakers_dict.__len__

    @property
    def update(self):
        for speaker in self.sonos_speakers:
            speaker.update()

    ##TODO refactor

    """Gets the requested speaker index of all speakers.
    """

    def get_speaker_index(self, speakers):
        stream_speakers_object = []

        for index, sonos_speaker in enumerate(self.sonos_speakers_dict):
            speaker_name = sonos_speaker['player_name']
            if speaker_name == speakers[0]:
                _LOGGER.debug(index, sonos_speaker)
                stream_speakers_object.append(index)

        return stream_speakers_object

    def play_stream(self, sonos_speakers_stream):
        for i in self.get_speaker_index(sonos_speakers_stream):
            self.sonos_speakers[i].play_stream()

    def play_party(self, sonos_speakers_stream):                
        _LOGGER.debug("Requested party play")

    def pause(self, sonos_speakers_stream):
        for i in self.get_speaker_index(sonos_speakers_stream):
            self.sonos_speakers[i].play()

    def stop(self, sonos_speakers_stream):
        for i in self.get_speaker_index(sonos_speakers_stream):
            self.sonos_speakers[i].stop()

    def stop(self, sonos_speakers_stream):
        for i in self.get_speaker_index(sonos_speakers_stream):
            self.sonos_speakers[i].stop()

    def set_volume(self, sonos_speakers_stream):
        for i in self.get_speaker_index(sonos_speakers_stream):
            self.sonos_speakers[i].set_volume()

    def reprJSON(self):
        return dict(sonos_speakers=self.sonos_speakers_dict)



import logging
import threading
import time
import argparse

from soundbridgefx import (
    ffmpeg,
    ledfx,
    raspotify,
    web_backend
)

def setup_logging(loglevel):
    console_loglevel = loglevel or logging.WARNING
    console_logformat = "[%(levelname)-8s] %(name)-30s : %(message)s"

    root_logger = logging.getLogger()

    console_handler = logging.StreamHandler()
    console_handler.setLevel(console_loglevel)  # set loglevel
    console_formatter = logging.Formatter(
        console_logformat
    )  # a simple console format
    console_handler.setFormatter(
        console_formatter
    )  # tell the console_handler to use this format

    # add the handlers to the root logger
    root_logger.setLevel(logging.DEBUG)
    root_logger.addHandler(console_handler)

    # Suppress some of the overly verbose logs
    logging.getLogger("sacn").setLevel(logging.WARNING)
    logging.getLogger("aiohttp.access").setLevel(logging.WARNING)
    logging.getLogger("pyupdater").setLevel(logging.WARNING)
    logging.getLogger("zeroconf").setLevel(logging.WARNING)

    global _LOGGER
    _LOGGER = logging.getLogger(__name__)


def main():
    # args = parse_args()
    # setup_logging(args.loglevel)

    ledfx_thread = threading.Thread(
        target=ledfx.main(), args=(1,), daemon=True)
    ledfx_thread.start()


    ffmpeg_thread = threading.Thread(
        target=ffmpeg.main(), args=(1,), daemon=True)
    ffmpeg_thread.start()

    # raspotify_thread = threading.Thread(
    #     target=raspotify.main(), args=(1,), daemon=True)
    # raspotify_thread.start()

    backend_thread = threading.Thread(target=web_backend.main(), args=(1,))
    backend_thread.start()

    try:
        while True:
            time.sleep(2)
    except KeyboardInterrupt:
        ffmpeg_thread.join()
        # ledfx_thread.join()
        # raspotify_thread.join()
        backend_thread.join()


import argparse
def parse_args():
    parser = argparse.ArgumentParser(
        description="A Networked LED Effect Controller"
    )

    parser.add_argument(
        "--version",
        action="version",
        version=f"soundbridgefx ",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        dest="loglevel",
        help="set loglevel to INFO",
        action="store_const",
        const=logging.INFO,
    )
    parser.add_argument(
        "-vv",
        "--very-verbose",
        dest="loglevel",
        help="set loglevel to DEBUG",
        action="store_const",
        const=logging.DEBUG,
    )
    parser.add_argument(
        "-lp",
        "--lport",
        dest="port",
        help="LedFx Web interface port (HTTP)",
        default=None,
        type=int,
    )
    parser.add_argument(
        "-lp_s",
        "--lport_secure",
        dest="port_s",
        help="LedFX Web interface port (HTTPS)",
        default=None,
        type=int,
    )
    parser.add_argument(
        "-sp",
        "--sport",
        dest="port",
        help="SoundBridgeFx Web interface port (HTTP)",
        default=None,
        type=int,
    )
    parser.add_argument(
        "-sp_s",
        "--sport_secure",
        dest="port_s",
        help="SoundBridgeFx Web interface port (HTTPS)",
        default=None,
        type=int,
    )
    parser.add_argument(
        "-mp",
        "--mport",
        dest="port",
        help="Stream port (HTTP)",
        default=None,
        type=int,
    )
    parser.add_argument(
        "-mp_s",
        "--mport_secure",
        dest="port_s",
        help="Stream port (HTTPS)",
        default=None,
        type=int,
    )

    return parser.parse_args()


import subprocess
import soundcard as sc
import logging

_LOGGER = logging.getLogger(__name__)


# class FFMPEG:

#     def __init__(self) -> None:
#         print("test")
#         self.speakers = sc.all_speakers()
#         self.microphones = sc.all_microphones()
#         self.current_speaker = sc.default_speaker()
#         self.current_microphone = sc.default_microphone()
#         pass

#     def get_all_speakers(self):
#         self.speakers
    
#     def get_all_micrphones(self):
#         self.microphones
    
#     def get_current_speaker(self):
#         self.current_speaker

#     def get_current_micrphone(self):
#         self.current_microphone
    

def main():

    # default_speaker = self.current_speaker
    try:
        p = subprocess.Popen(
            "ffmpeg -f alsa -ac 2 -i default -acodec libmp3lame -ab 256k -f mp3 stream.mp3",
            stdout=subprocess.PIPE,
            shell=True)
    except:
        _LOGGER.error("Could not start ffmpeg")


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['soundbridgefx']

package_data = \
{'': ['*'],
 'soundbridgefx': ['web/client/*',
                   'web/client/_app/*',
                   'web/client/_app/immutable/assets/*',
                   'web/client/_app/immutable/chunks/*',
                   'web/client/_app/immutable/entry/*',
                   'web/prerendered/pages/*']}

install_requires = \
['argparse>=1.4.0,<2.0.0',
 'flask-cors>=3.0.10,<4.0.0',
 'flask>=2.2.2,<3.0.0',
 'jsonpickle>=3.0.1,<4.0.0',
 'ledfx>=2.0.60,<3.0.0',
 'pyaudio>=0.2.13,<0.3.0',
 'soco>=0.28.1,<0.29.0',
 'soundcard>=0.4.2,<0.5.0']

entry_points = \
{'console_scripts': ['soundbridgefx = soundbridgefx.__main__:main']}

setup_kwargs = {
    'name': 'soundbridgefx',
    'version': '0.2.1b0',
    'description': '',
    'long_description': '# SoundBridge\nSpotify LedFx Sonos Proxy\n\n## About\n\nThis project tries to make an Spotify LedFX Sonos proxy, meaning it will show up as an spotify connect device allowing you to stream music to this device.\nThen the "proxy" part will kick in and will get the incoming spotify audio to both ledfx and a sonos device of your chosing, allowing you to headless hear and see your music.\n\nCurrently WIP',
    'author': 'Martyn van Dijke',
    'author_email': 'martijnvdijke600@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/martynvdijke/SoundBridgeFx',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0',
}


setup(**setup_kwargs)
